@@ -3,8 +3,10 @@
 //
 // Author:
 //   Aaron Bockover <abockover@novell.com>
+//   Julien Moutte <julien@fluendo.com>
 //
 // Copyright (C) 2005-2008 Novell, Inc.
+// Copyright (C) 2010 Fluendo S.A.
 //
 // Permission is hereby granted, free of charge, to any person obtaining
 // a copy of this software and associated documentation files (the
@@ -38,14 +40,14 @@
 #include <gst/base/gstadapter.h>
 #include <gdk/gdk.h>
 #include <gst/fft/gstfftf32.h>
+#include <gst/pbutils/pbutils.h>
 
-#ifdef HAVE_GST_PBUTILS
-#  include <gst/pbutils/pbutils.h>
-#endif
-
-#ifdef GDK_WINDOWING_X11
+#if defined(GDK_WINDOWING_X11)
 #  include <gdk/gdkx.h>
 #  include <gst/interfaces/xoverlay.h>
+#elif defined(GDK_WINDOWING_WIN32)
+#  include <gdk/gdkwin32.h>
+#  include <gst/interfaces/xoverlay.h>
 #endif
 
 #include "banshee-gst.h"
@@ -61,11 +63,28 @@
 #define IS_BANSHEE_PLAYER(e) (e != NULL)
 #define SET_CALLBACK(cb_name) { if(player != NULL) { player->cb_name = cb; } }
 
+#define BANSHEE_CHECK_GST_VERSION(major,minor,micro) \
+    (GST_VERSION_MAJOR > (major) || \
+        (GST_VERSION_MAJOR == (major) && GST_VERSION_MINOR > (minor)) || \
+        (GST_VERSION_MAJOR == (major) && GST_VERSION_MINOR == (minor) && \
+            GST_VERSION_MICRO >= (micro)))
+
+#if BANSHEE_CHECK_GST_VERSION(0,10,25)
+#include <gst/interfaces/streamvolume.h>
+#endif
+
 #ifdef WIN32
-// TODO Windows doesn't like the ... varargs
-#define bp_debug(x)
+#define bp_debug(x) banshee_log_debug ("player", x)
+#define bp_debug2(x, a2) banshee_log_debug ("player", x, a2)
+#define bp_debug3(x, a2, a3) banshee_log_debug ("player", x, a2, a3)
+#define bp_debug4(x, a2, a3, a4) banshee_log_debug ("player", x, a2, a3, a4)
+#define bp_debug5(x, a2, a3, a4, a5) banshee_log_debug ("player", x, a2, a3, a4, a5)
 #else
 #define bp_debug(x...) banshee_log_debug ("player", x)
+#define bp_debug2(x...) banshee_log_debug ("player", x)
+#define bp_debug3(x...) banshee_log_debug ("player", x)
+#define bp_debug4(x...) banshee_log_debug ("player", x)
+#define bp_debug5(x...) banshee_log_debug ("player", x)
 #endif
 
 typedef struct BansheePlayer BansheePlayer;
@@ -79,7 +98,12 @@ typedef void (* BansheePlayerIterateCallback)      (BansheePlayer *player);
 typedef void (* BansheePlayerBufferingCallback)    (BansheePlayer *player, gint buffering_progress);
 typedef void (* BansheePlayerTagFoundCallback)     (BansheePlayer *player, const gchar *tag, const GValue *value);
 typedef void (* BansheePlayerVisDataCallback)      (BansheePlayer *player, gint channels, gint samples, gfloat *data, gint bands, gfloat *spectrum);
+typedef void (* BansheePlayerNextTrackStartingCallback)     (BansheePlayer *player);
+typedef void (* BansheePlayerAboutToFinishCallback)         (BansheePlayer *player);
 typedef GstElement * (* BansheePlayerVideoPipelineSetupCallback) (BansheePlayer *player, GstBus *bus);
+typedef void (* BansheePlayerVideoPrepareWindowCallback) (BansheePlayer *player);
+typedef void (* BansheePlayerVolumeChangedCallback) (BansheePlayer *player, gdouble new_volume);
+typedef void (* BansheePlayerVideoGeometryNotifyCallback) (BansheePlayer *player, gint width, gint height, gint fps_n, gint fps_d, gint par_n, gint par_d);
 
 typedef enum {
     BP_VIDEO_DISPLAY_CONTEXT_UNSUPPORTED = 0,
@@ -96,7 +120,12 @@ struct BansheePlayer {
     BansheePlayerBufferingCallback buffering_cb;
     BansheePlayerTagFoundCallback tag_found_cb;
     BansheePlayerVisDataCallback vis_data_cb;
+    BansheePlayerNextTrackStartingCallback next_track_starting_cb;
+    BansheePlayerAboutToFinishCallback about_to_finish_cb;
     BansheePlayerVideoPipelineSetupCallback video_pipeline_setup_cb;
+    BansheePlayerVideoPrepareWindowCallback video_prepare_window_cb;
+    BansheePlayerVolumeChangedCallback volume_changed_cb;
+    BansheePlayerVideoGeometryNotifyCallback video_geometry_notify_cb;
 
     // Pipeline Elements
     GstElement *playbin;
@@ -104,22 +133,44 @@ struct BansheePlayer {
     GstElement *audiobin;
     GstElement *equalizer;
     GstElement *preamp;
+    GstElement *volume;
+    GstElement *rgvolume;
+
+    GstElement *before_rgvolume;
+    GstElement *after_rgvolume;
+    gboolean   rgvolume_in_pipeline;
+
     gint equalizer_status;
     gdouble current_volume;
     
     // Pipeline/Playback State
-    GMutex *mutex;
+    GMutex *video_mutex;
+    GMutex *replaygain_mutex;
     GstState target_state;
-    guint iterate_timeout_id;
     gboolean buffering;
     gchar *cdda_device;
+    gboolean in_gapless_transition;
+    gboolean supports_stream_volume;
+    gboolean audiosink_has_volume;
     
     // Video State
     BpVideoDisplayContextType video_display_context_type;
-    #ifdef GDK_WINDOWING_X11
+    #if defined(GDK_WINDOWING_X11)
     GstXOverlay *xoverlay;
     GdkWindow *video_window;
+    XID video_window_xid;
+    #elif defined(GDK_WINDOWING_WIN32)
+    GstXOverlay *xoverlay;
+    GdkWindow *video_window;
+    HWND video_window_xid;
     #endif
+    // Video geometry
+    gint width;
+    gint height;
+    gint fps_n;
+    gint fps_d;
+    gint par_n;
+    gint par_d;
        
     // Visualization State
     GstElement *vis_resampler;
@@ -135,9 +186,7 @@ struct BansheePlayer {
     GSList *missing_element_details;
     GSList *missing_element_details_handled;
     gboolean handle_missing_elements;
-    #ifdef HAVE_GST_PBUTILS
     GstInstallPluginsContext *install_plugins_context;
-    #endif
     
     // ReplayGain State
     gboolean replaygain_enabled;
@@ -147,15 +196,8 @@ struct BansheePlayer {
     // and the oldest at index 10. History is used to compute 
     // gain on a track where no adjustment information is present.
     // http://replaygain.hydrogenaudio.org/player_scale.html
-    gdouble volume_scale_history[11];
-    gboolean volume_scale_history_shift;
-    gboolean current_scale_from_history;
-    
-    // ReplayGain cache
-    gdouble album_gain;
-    gdouble album_peak;
-    gdouble track_gain;
-    gdouble track_peak;
+    gdouble rg_gain_history[10];
+    gint history_size;
 };
 
 #endif /* _BANSHEE_PLAYER_PRIVATE_H */
