@@ -49,7 +49,11 @@ namespace Banshee.Database
             get { return configuration; }
         }
 
-        public BansheeDbConnection () : base (DatabaseFile)
+        public BansheeDbConnection () : this (DatabaseFile)
+        {
+        }
+
+        internal BansheeDbConnection (string db_path) : base (db_path)
         {
             // Each cache page is about 1.5K, so 32768 pages = 49152K = 48M
             int cache_size = (TableExists ("CoreTracks") && Query<long> ("SELECT COUNT(*) FROM CoreTracks") > 10000) ? 32768 : 16384;
@@ -58,17 +62,19 @@ namespace Banshee.Database
             Execute ("PRAGMA temp_store = MEMORY");
             Execute ("PRAGMA count_changes = OFF");
 
-            // don't want this on because it breaks searching/smart playlists.  See BGO #526371
+            // TODO didn't want this on b/c smart playlists used to rely on it, but
+            // now they shouldn't b/c we have smart custom functions we use for sorting/searching.
+            // See BGO #603665 for discussion about turning this back on.
             //Execute ("PRAGMA case_sensitive_like=ON");
 
-            Log.DebugFormat ("Opened SQLite connection to {0}", DatabaseFile);
+            Log.DebugFormat ("Opened SQLite (version {1}) connection to {0}", db_path, ServerVersion);
 
             migrator = new BansheeDbFormatMigrator (this);
             configuration = new DatabaseConfigurationClient (this);
-            
-            if (Banshee.Base.ApplicationContext.CommandLine.Contains ("debug-sql")) {
+
+            if (ApplicationContext.CommandLine.Contains ("debug-sql")) {
                 Hyena.Data.Sqlite.HyenaSqliteCommand.LogAll = true;
-                WarnIfCalledFromThread = Banshee.Base.ThreadAssist.MainThread;
+                WarnIfCalledFromThread = ThreadAssist.MainThread;
             }
         }
 
@@ -83,9 +89,19 @@ namespace Banshee.Database
                 } catch (Exception e) {
                     Log.Exception ("Error determining if ANALYZE is necessary", e);
                 }
-                
+
                 // Update cached sorting keys
-                SortKeyUpdater.Update ();
+                BeginTransaction ();
+                try {
+                    SortKeyUpdater.Update ();
+                    CommitTransaction ();
+                } catch {
+                    RollbackTransaction ();
+                }
+            }
+
+            if (Banshee.Metrics.BansheeMetrics.EnableCollection.Get ()) {
+                Banshee.Metrics.BansheeMetrics.Start ();
             }
         }
 
@@ -96,7 +112,7 @@ namespace Banshee.Database
             string [] tables_with_indexes = {"CoreTracks", "CoreArtists", "CoreAlbums",
                 "CorePlaylistEntries", "PodcastItems", "PodcastEnclosures",
                 "PodcastSyndications", "CoverArtDownloads"};
-            
+
             if (TableExists ("sqlite_stat1")) {
                 foreach (string table_name in tables_with_indexes) {
                     if (TableExists (table_name)) {
@@ -115,13 +131,13 @@ namespace Banshee.Database
             } else {
                 needs_analyze = true;
             }
-            
+
             if (needs_analyze) {
                 Log.DebugFormat ("Running ANALYZE against database to improve performance");
                 Execute ("ANALYZE");
             }
         }
- 
+
         public BansheeDbFormatMigrator Migrator {
             get { lock (this) { return migrator; } }
         }
@@ -131,16 +147,16 @@ namespace Banshee.Database
                 if (ApplicationContext.CommandLine.Contains ("db")) {
                     return ApplicationContext.CommandLine["db"];
                 }
-                
+
                 string proper_dbfile = Path.Combine (Paths.ApplicationData, "banshee.db");
                 if (File.Exists (proper_dbfile)) {
                     return proper_dbfile;
                 }
 
                 string dbfile = Path.Combine (Path.Combine (Environment.GetFolderPath (
-                    Environment.SpecialFolder.ApplicationData), 
-                    "banshee"), 
-                    "banshee.db"); 
+                    Environment.SpecialFolder.ApplicationData),
+                    "banshee"),
+                    "banshee.db");
 
                 if (!File.Exists (dbfile)) {
                     string tdbfile = Path.Combine (Path.Combine (Path.Combine (Environment.GetFolderPath (
@@ -151,12 +167,12 @@ namespace Banshee.Database
 
                     dbfile = tdbfile;
                 }
-                
+
                 if (File.Exists (dbfile)) {
                     Log.InformationFormat ("Copying your old Banshee Database to {0}", proper_dbfile);
                     File.Copy (dbfile, proper_dbfile);
                 }
-                
+
                 return proper_dbfile;
             }
         }
