@@ -214,7 +214,8 @@ namespace Banshee.MediaEngine
         protected virtual void OnEventChanged (PlayerEventArgs args)
         {
             if (args.Event == PlayerEvent.StartOfStream && pending_track != null) {
-                Log.DebugFormat ("OnEventChanged called with StartOfStream.  Replacing current_track: \"{0}\" with pending_track: \"{1}\"", current_track.DisplayTrackTitle, pending_track.DisplayTrackTitle);
+                Log.DebugFormat ("OnEventChanged called with StartOfStream.  Replacing current_track with pending_track: \"{0}\"",
+                                 pending_track.DisplayTrackTitle);
                 current_track = pending_track;
                 current_uri = pending_uri;
                 pending_track = null;
@@ -242,7 +243,8 @@ namespace Banshee.MediaEngine
 
         protected void OnTagFound (StreamTag tag)
         {
-            if (tag.Equals (StreamTag.Zero) || current_track == null || current_track.Uri.IsFile) {
+            if (tag.Equals (StreamTag.Zero) || current_track == null
+                || (current_track.Uri != null && current_track.Uri.IsFile)) {
                 return;
             }
 
