@@ -337,10 +337,12 @@ namespace Banshee.Dap
 
         protected bool TrackNeedsTranscoding (TrackInfo track)
         {
-            foreach (string mimetype in AcceptableMimeTypes) {
-                if (ServiceManager.MediaProfileManager.GetExtensionForMimeType (track.MimeType) ==
-                    ServiceManager.MediaProfileManager.GetExtensionForMimeType (mimetype)) {
-                    return false;
+            string extension = ServiceManager.MediaProfileManager.GetExtensionForMimeType (track.MimeType);
+            if (extension != null) {
+                foreach (string mimetype in AcceptableMimeTypes) {
+                    if (extension == ServiceManager.MediaProfileManager.GetExtensionForMimeType (mimetype)) {
+                        return false;
+                    }
                 }
             }
 
